import React, { useCallback, useEffect, useRef, useState } from 'react';
import { filterData, paginateData, sortData } from '../helpers/data';
import { convertArrayToRecord } from '../helpers/object';
import { createCtx } from '../helpers/react';
const [useCtx, Provider] = createCtx();
export const useDatatableWrapper = useCtx;
export function DatatableWrapper({ headers, body, filterProps, sortProps, paginationProps, paginationOptionsProps, children }) {
    const [state, setState] = useState(getDefaultDatatableState({
        filterProps,
        headers,
        sortProps,
        paginationProps,
        paginationOptionsProps
    }));
    const tableHeadersRecordRef = useRef(convertArrayToRecord(headers, 'prop'));
    // Store this in a ref because we might need it in case `headers` change.
    // Though, we don't want to put these into `useEffect` because they most likely
    // change over time.
    const controlPropsRef = useRef({
        filterProps,
        sortProps,
        paginationProps,
        paginationOptionsProps
    });
    useEffect(() => {
        // Resets the table if the headers passed down is different.
        if (headers !== undefined) {
            // Re-set the ref.
            tableHeadersRecordRef.current = convertArrayToRecord(headers, 'prop');
            // Re-set the initial state.
            setState(getDefaultDatatableState({
                filterProps: controlPropsRef.current.filterProps,
                headers,
                sortProps: controlPropsRef.current.sortProps,
                paginationProps: controlPropsRef.current.paginationProps,
                paginationOptionsProps: controlPropsRef.current.paginationOptionsProps
            }));
        }
    }, [headers]);
    useEffect(() => {
        // Resets the table if the data passed down is different.
        if (body !== undefined) {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { filter: '', pagination: Object.assign(Object.assign({}, oldState.pagination), { currentPage: 1 }) })));
        }
    }, [body]);
    const onFilterChange = useCallback((text) => {
        setState((oldState) => (Object.assign(Object.assign({}, oldState), { filter: text, pagination: Object.assign(Object.assign({}, oldState.pagination), { currentPage: 1 }) })));
    }, []);
    const onPaginationChange = useCallback((nextPage) => {
        setState((oldState) => (Object.assign(Object.assign({}, oldState), { pagination: Object.assign(Object.assign({}, oldState.pagination), { currentPage: nextPage }) })));
    }, []);
    const onRowsPerPageChange = useCallback((numOfPage) => {
        setState((oldState) => (Object.assign(Object.assign({}, oldState), { pagination: Object.assign(Object.assign({}, oldState.pagination), { rowsPerPage: numOfPage, currentPage: 1 }) })));
    }, []);
    const onSortChange = useCallback((nextSort) => {
        setState((oldState) => (Object.assign(Object.assign({}, oldState), { sort: nextSort })));
    }, []);
    const { filter, sort, pagination } = state;
    let data = body;
    let filteredDataLength = data.length;
    let maxPage = 1;
    if (state.isFilterable) {
        data = filterData(body, tableHeadersRecordRef.current, filter, filterProps === null || filterProps === void 0 ? void 0 : filterProps.filterValueObj);
        filteredDataLength = data.length;
    }
    data = sortData(data, sort, sortProps === null || sortProps === void 0 ? void 0 : sortProps.sortValueObj);
    if (pagination.rowsPerPage !== -1) {
        // Pagination needs.
        maxPage = Math.ceil(data.length / pagination.rowsPerPage);
        data = paginateData(data, pagination.currentPage, pagination.rowsPerPage);
    }
    return (React.createElement(Provider, { value: {
            headers,
            // Filter.
            isFilterable: state.isFilterable,
            filterState: state.filter,
            onFilterChange,
            // Sort.
            sortState: state.sort,
            onSortChange,
            // Pagination.
            currentPageState: state.pagination.currentPage,
            onPaginationChange,
            // Pagination options.
            rowsPerPageState: state.pagination.rowsPerPage,
            rowsPerPageOptions: state.pagination.rowsPerPageOptions,
            onRowsPerPageChange,
            // Data.
            maxPage,
            filteredDataLength,
            data
        } }, children));
}
// Helper functions.
function getDefaultDatatableState({ filterProps, paginationOptionsProps, paginationProps, sortProps, headers }) {
    var _a, _b, _c, _d, _e;
    const defaultSort = { order: 'asc', prop: '' };
    let isFilterable = filterProps !== undefined;
    for (const header of headers) {
        // Set the default sort header.
        if (
        // If the sorted prop is still "empty", then we assign it with the current header.
        (defaultSort.prop === '' && header.isSortable) ||
            // Or, if the header prop matches the initial state, then set it as well.
            header.prop === ((_a = sortProps === null || sortProps === void 0 ? void 0 : sortProps.initialState) === null || _a === void 0 ? void 0 : _a.prop)) {
            defaultSort.prop = header.prop.toString();
        }
        if (header.isFilterable) {
            isFilterable = true;
        }
    }
    // Define initial state.
    return {
        isFilterable,
        filter: ((_b = filterProps === null || filterProps === void 0 ? void 0 : filterProps.initialState) === null || _b === void 0 ? void 0 : _b.filter) || '',
        sort: defaultSort,
        pagination: {
            currentPage: ((_c = paginationProps === null || paginationProps === void 0 ? void 0 : paginationProps.initialState) === null || _c === void 0 ? void 0 : _c.page) || 1,
            rowsPerPage: ((_d = paginationOptionsProps === null || paginationOptionsProps === void 0 ? void 0 : paginationOptionsProps.initialState) === null || _d === void 0 ? void 0 : _d.rowsPerPage) || -1,
            rowsPerPageOptions: ((_e = paginationOptionsProps === null || paginationOptionsProps === void 0 ? void 0 : paginationOptionsProps.initialState) === null || _e === void 0 ? void 0 : _e.options) || []
        }
    };
}
